# -*- coding: utf-8 -*-
"""
Created on Thu Nov  3 11:58:45 2022
@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

# Standard toolboxes
import numpy as np
import matplotlib.pyplot as plt
import sys
import os

# Allow for subsampling of large datasets for plotting & saving as PDF
sys.path.append('functions')
from functions.subsample_routines import subsample_binned_2D
from functions.dedicated_screening_functions import calc_SQI_orig

# Close all figures
plt.close('all')


#############
# START INPUT
#############

# Indicator quality case (to load the right case study dataset). 
study = 'VBM'
# study = 'WAVES'

# Folder name in which to store figures (rel. to present folder)
figfolder = 'out_figs_paper_PRADS'

# Plot orientation (only used when there are 2 cases defined above)
orient = 'hor'
# orient = 'ver'

if study == 'VBM':

    inpcases = ['Hogging','Sagging']
    
    # Folder name where input files are stored (files to read)
    inputfolder = 'inputfiles_vbm'
    
    # Maximum plot limit for HF input [Nm]
    maxvalinHF = 1.6001e9
    stepsizeHF = 0.4e9
    
elif study == 'WAVES':
    
    inpcases = ['GoodInd','WorseInd']
    
    # Folder name where input files are stored (files to read)
    inputfolder = 'inputfiles_waves'
    
    # Maximum plot limit for HF input [m]
    maxvalinHF = 14.001
    stepsizeHF = 2
    
    # Number of seeds (this is in the name of the input file)
    Ns = 50
    

#############
# END INPUT
#############

# Check wether output directories exist, if not make them
isExist = os.path.exists(figfolder)
if not isExist:
   os.makedirs(figfolder)

# Initialise
colgrey = [0.7,0.7,0.7]
if len(inpcases) == 4:
    fig7 = plt.figure(7,figsize=(4.8,3.75))
    fig8 = plt.figure(8,figsize=(4.8,3.75))
elif len(inpcases) == 2 and orient == 'hor':
    # fig7 = plt.figure(7,figsize=(4.8,1.8))
    fig7 = plt.figure(7,figsize=(4,1.8))
    fig8 = plt.figure(8,figsize=(4.8,1.8))
elif len(inpcases) == 2 and orient == 'ver':
    fig7 = plt.figure(7,figsize=(2.4,3.75))
    fig8 = plt.figure(8,figsize=(2.4,3.75))
xticksnew = np.arange(0,maxvalinHF,stepsizeHF)
yticksnew = np.arange(0,maxvalinHF,stepsizeHF)

for nn,inpcase in enumerate(inpcases):
    
    
    if inpcase == 'Sagging':
        
        # HF symbols and units
        # HFsymb = 'V_{sag} \prime\prime'
        HFsymb = "V''"
        HFunit = 'Nm'
        
        # Indicator cases
        indcase = 'LF sagging' 
        # indalias = 'V_{sag}\prime'
        indalias = "V'"
        indunit = 'Nm'
       
    elif inpcase == 'Hogging':

        # HF symbols and units
        # HFsymb = 'V_{hog} \prime\prime'
        HFsymb = "V''"
        HFunit = 'Nm'
        
        # Indicator cases
        indcase = 'LF hogging'
        # indalias = 'V_{hog}\prime'
        indalias = "V'"
        indunit = 'Nm'
        
    elif inpcase == 'GoodInd':
        
        # HF symbols and units
        HFsymb = "C''"
        HFunit = 'm'
        
        # Indicator cases
        indcase = 'LF GoodInd' 
        indalias = "C'_{good}"
        indunit = 'm'
            
    elif inpcase == 'WorseInd':
        
        # HF symbols and units
        HFsymb = "C''"
        HFunit = 'm'
        
        # Indicator cases
        indcase = 'LF WorseInd' 
        indalias = "C'_{worse}"
        indunit = 'm'

    if study == 'VBM':
        # Read true MCS time trace data from CSV
        AllTrueDat_scr = np.loadtxt(f'{inputfolder}/MARINferry2_MatchedPeaks{inpcase}_VBM_JONS_Hs13.2m_Tp10.0s_gam3.0_Vs10.00kn_mu180deg.csv', skiprows=1, delimiter=',')
        # t_peaks_lin_all = AllTrueDat_scr[:,0]
        peaks_lin_all = AllTrueDat_scr[:,1]
        # t_peaks_2nd_mat = AllTrueDat_scr[:,2]
        peaks_2nd_mat = AllTrueDat_scr[:,3]
        
    elif study == 'WAVES':
        
        # Read matched LF and HF peaks from CSV file
        AllTrueDat_scr = np.loadtxt(f'{inputfolder}/Lin2nd_trueLFandHFdata_{inpcase}_{Ns}seeds_scr.csv', delimiter=',')
        # t_peaks_lin_all = AllTrueDat_scr[:,0]
        peaks_lin_all = AllTrueDat_scr[:,1]
        # t_peaks_2nd_mat = AllTrueDat_scr[:,2]
        peaks_2nd_mat = AllTrueDat_scr[:,3]
        
    # Subsample for the scatter plots
    xbins = np.arange(0,maxvalinHF,maxvalinHF/50)
    ybins = np.arange(0,maxvalinHF,maxvalinHF/50)
    peaks_lin_all_samp,peaks_2nd_all_samp = subsample_binned_2D(peaks_lin_all,peaks_2nd_mat,xbins,ybins)
    
    # Scatter plot indicator vs NL response peaks and selected samples
    plt.figure(7)
    if len(inpcases) == 4:
        plt.subplot(2,2,nn+1)
    elif len(inpcases) == 2 and orient == 'hor':
        plt.subplot(1,2,nn+1)
    elif len(inpcases) == 2 and orient == 'ver':
        plt.subplot(2,1,nn+1)
    plt.plot(peaks_lin_all_samp,peaks_2nd_all_samp,marker='o',color=colgrey,label='All peaks',linestyle = 'None')
    plt.plot(np.array([0,maxvalinHF]),np.array([0,maxvalinHF]),'--',color='grey')
    if len(inpcases) == 4:
        if nn > 1:
            plt.xticks(xticksnew)
        else:
            plt.xticks(xticksnew,labels=[])
        if nn == 0 or nn == 2:
            # plt.ylabel('HF response: 2$^{nd}$-\n order wave crests [m]')
            plt.yticks(yticksnew)
        else:
            plt.yticks(yticksnew,labels=[])
    elif len(inpcases) == 2 and orient == 'hor':
        plt.xticks(xticksnew)
        if nn == 0:
            plt.yticks(yticksnew)
        else:
            plt.yticks(yticksnew,labels=[])
    elif len(inpcases) == 2 and orient == 'ver':
        plt.yticks(yticksnew) 
        if nn == 1:
            plt.xticks(xticksnew)
        else:
            plt.xticks(xticksnew,labels=[])
         
    plt.xlim([0,maxvalinHF])
    plt.ylim([0,maxvalinHF])
    plt.xlabel(r'$%s$ [%s]' % (indalias,indunit))
    if nn == 0:
        plt.ylabel(r'$%s$ [%s]' %(HFsymb,HFunit))
    # plt.title(f'{indalias}')
    
    # Threshold range for SQI calculation
    thr_range = np.linspace(0,maxvalinHF,100)
    
    # Calculate SQI over threshold range
    SQI = calc_SQI_orig(peaks_lin_all,peaks_2nd_mat,thr_range)
    
    plt.figure(8)
    if orient == 'hor':
        plt.subplot(1,2,nn+1)
    elif orient == 'ver':
        plt.subplot(2,1,nn+1)
    plt.plot(thr_range,SQI)
    plt.xlabel(r'Threshold $%s$ [%s]' % (HFsymb, HFunit))
    if nn == 0:
        plt.ylabel('SQI [-]')
    plt.title(r'Indicator: $%s$' %(indalias))
    plt.xlim([0,maxvalinHF])
    plt.ylim([0,10])
    plt.yticks([0,2,4,6,8,10])
    
    # Calculate SQI10 (same as in OE paper; using the SQI at the threshold
    # corresponding to the 10th highest screened HF peak)
    HFsorted = np.sort(peaks_2nd_mat)
    HF10th = HFsorted[-10]
    SQI10 = np.interp(HF10th,thr_range,SQI)
    
    plt.figure(8)
    plt.plot([HF10th,HF10th],[0,SQI10],'--k')
    plt.text(HF10th*1.02,0.5*SQI10,'10th\nmax. peak')
    plt.plot([0,HF10th],[SQI10,SQI10],'--k')
    plt.text(HF10th*0.1,1.1*SQI10,'SQI10')
    
    print(f'Indicator: {indalias}')
    print(f'10th highest HF peak: {HF10th} {HFunit}')
    print(f'SQI10: {SQI10}')
    
    # Calculate mean SQI over full range (same as in the OE paper)
    SQImean = np.nanmean(SQI)
    print(f'SQImean: {SQImean}')
    
    if nn == 0:
        plt.figure(7)
        plt.savefig(f'{figfolder}/Scatters_{study}_{inpcase}.pdf',bbox_inches='tight')
    
        
    

# # Add shared axes labels and save figure
# if len(indcases) == 4:
#     fig7.text(0.5, 0.02, 'LF indicator: linear wave crests [m]', ha='center')
#     fig7.text(0.03, 0.5, 'HF response: 2$^{nd}$-order wave crests [m]', va='center', rotation='vertical')
# elif len(indcases) == 2 and orient == 'hor':
#     fig7.text(0.5, -0.1, 'LF indicator crests [m]', ha='center')
#     fig7.text(0.03, 0.5, 'HF response crests [m]', va='center', rotation='vertical')
# elif len(indcases) == 2 and orient == 'ver':
#     fig7.text(0.5, 0.02, 'LF indicator crests [m]', ha='center')
#     fig7.text(-0.05, 0.5, 'HF response crests [m]', va='center', rotation='vertical')
plt.figure(7)
plt.savefig(f'{figfolder}/Scatters_{study}.pdf',bbox_inches='tight')
plt.figure(8)
plt.savefig(f'{figfolder}/SQI_{study}.pdf',bbox_inches='tight')

