# -*- coding: utf-8 -*-
"""
Created on Thu Apr 18 15:30:02 2024

@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

import numpy as np
import matplotlib.pyplot as plt
from scipy.stats import genpareto
from scipy.optimize import curve_fit
from scipy.optimize import least_squares

# Close all figures
plt.close('all')

# shape = # shape parameter (xi)
# scale = # scale parameter (sigma)
# loc = # location parameter (mu)

# CDF definition
def GP_cdf(x,loc,scale,shape):
    if shape == 0:
        F_gp = 1-np.exp(-(x-loc)/scale)
    else:
        F_gp = 1-(1+(shape*(x-loc))/scale)**(-1/shape)    
    return F_gp

# PDF definition
def GP_pdf(x,loc,scale,shape):
    if shape == 0:
        f_gp = np.exp(-x)    
    else:
        f_gp = (1/scale)*(1+(shape*(x-loc))/scale)**(-1/shape-1)
    return f_gp

# Probability of exceedance distribution definition
def GP_pexc(x, loc, scale, shape):
    if shape == 0:
        F_gp = 1-np.exp(-(x-loc)/scale)
    else:
        F_gp = 1-(1+(shape*(x-loc))/scale)**(-1/shape)    
    pexc = 1-F_gp
    return pexc

# Log of probability of exceedance distribution definition
def GP_pexc_log(x, loc, scale, shape):
    if shape == 0:
        F_gp = 1-np.exp(-(x-loc)/scale)
    else:
        F_gp = 1-(1+(shape*(x-loc))/scale)**(-1/shape)    
    pexc = 1-F_gp
    pexc_log = np.log(pexc)
    return pexc_log

# Probability of exceedance distribution definition EXPONENTIAL DISTRIBUTION
def GPshape0_pexc(x, loc, scale):
    F_gp = 1-np.exp(-(x-loc)/scale)
    pexc = 1-F_gp
    return pexc

# Log of probability of exceedance distribution definition EXPONENTIAL DISTRIBUTION
def GPshape0_pexc_log(x, loc, scale):
    F_gp = 1-np.exp(-(x-loc)/scale)
    pexc = 1-F_gp
    pexc_log = np.log(pexc)
    return pexc_log



# def fit_gpd_cdf_to_empirical_cdf(V, P):
    
#     # Define the error function to minimize (sum of squared differences)
#     def error_function(params):
#         loc, scale, shape = params
#         fitted_cdf = GP_cdf(V, loc, scale, shape)
#         return np.sum((P - fitted_cdf) ** 2)

#     # Initial guess for parameters
#     initial_guess = [np.min(V), np.std(V), 0.5]

#     # Fit GPD CDF to empirical CDF using curve fitting
#     params, _ = curve_fit(GP_cdf, V, P, p0=initial_guess)

#     return params

# Function to calculate residuals for least squares fitting
def residuals(params, V, P):
    loc, scale, shape = params
    fitted_cdf = GP_cdf(V, loc, scale, shape)
    # return np.abs(np.log(P) - np.log(fitted_cdf))
    return P-fitted_cdf


def fit_gpd_cdf_to_empirical_cdf_least_squares(V, P, initial_guess):
    
    # # Sort V and P together based on the values in V - this does not help
    # sorted_VP = sorted(zip(V, P))
    # V, P = zip(*sorted_VP)
        
    V = np.squeeze(V)
    P = np.squeeze(P)
    
    # # Initial guess for parameters (loc,scale,shape / mu,sigma,xi)
    # initial_guess = [np.min(V), np.std(V), 0.5]
    
    # Define bounds for parameters (loc, scale, shape)
    # bounds = ([np.min(V), 0.001, -10], [np.max(V), np.max(V) - np.min(V), 10])
    bounds = ([0, 0.0001, -20], [np.max(V), 100, 20])

    # Fit GPD CDF to empirical CDF using least squares
    result = least_squares(residuals, initial_guess, bounds=bounds, args=(V, P))
    
    return result.x

# Function to fit Generalized Pareto Distribution (GPD) to peak over GP_threshold (POT) observations
def fit_gpd_to_pot(observations, GP_threshold):
    # Extract POT observations
    pot_observations = observations[observations > GP_threshold]

    # Fit GPD parameters using Maximum Likelihood Estimation (MLE)
    GP_params = genpareto.fit(pot_observations - GP_threshold)

    return GP_params

# Empirical CDF (newly ordered, used to debug the GP fit
def empirical_cdf(observations, GP_threshold):
    # Compute empirical CDF
    pot_observations = observations[observations > GP_threshold]
    pot_observations = np.sort(pot_observations)
    ecdf_values = np.arange(1, len(pot_observations) + 1) / len(pot_observations)
    return pot_observations, ecdf_values







