
import numpy as np


class MaxDist:
    """
    This acquisition function selects a new point at the location where all 
    other points are furthest away - so in the middle of the largest present 
    distance between the input samples.
    """
    
    def __init__(self,p_samples: float) -> None:
        self.p_samples = p_samples

    
    def evaluate(self, x: np.ndarray) -> np.ndarray:
        
        # For MF-GPR, reshape x
        if np.shape(x)[1] == 2:
            x = x[:,0]
        
        # Define y
        y = self.p_samples
        
        # Reshape arrays to ensure proper broadcasting
        X_reshaped = x.reshape(-1, 1)  # Reshape X to column vector
        Y_reshaped = y.reshape(1, -1)  # Reshape Y to row vector
        
        # Calculate distances using broadcasting
        distances = np.abs(X_reshaped - Y_reshaped)
        
        # Find minimum distance for each point in X
        min_distances = np.min(distances, axis=1)
        
        return min_distances
    
    @property
    def has_gradients(self):
        return False

