# -*- coding: utf-8 -*-
"""
Created on Wed Sep  6 16:47:18 2023

@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

# Standard toolboxes
import numpy as np
import matplotlib.pyplot as plt
import sys
import time

# Import custom functions
sys.path.append('functions')
# from functions.distribution_scripts import calc_distribution
from functions.Adaptive_Screening_v9 import adaptive_screening
from functions.T_enc import calc_T_enc

# Close all figures
plt.close('all')

# Record start time
starttime = time.time()

#############
# START INPUT
#############

# Number of considered iterations for GPR
maxitHF = 150

# Number of considered iterations for MF-GPR
maxitMF = 150
# maxitMF = 0 # to skip MF-GPR when we select IVR

# Folder name where input files are stored (files to read)
inputfolder = 'inputfiles_vbm'

# Output folder name in which to store files and figures (rel. to present folder)
filefolder = 'out_files_vbm_PRADS'
figfolder = 'out_figs_vbm_PRADS'

# Target exposure duration [s]
dur = 1800

# Total duration available data [s]
Tdata = 3600*30

# Wave properties (identical for both cases)
spec = 'JONS'
Hs = 13.2  # [m]
Tp = 10.0  # [s]
Vs = 10.00 # [kn]
mu = 180   # [deg]
gam = 3.0  # [-] 

# Initial sample option (see Adaptive Screening script)
init_sampopt = 'B'
# init_sampopt = 'A'

# Probabilty of exceedance (PoE) range for the prediction
# prange_low = 5*10**(-4) # 1h
# prange_high = 2*10**(-2) # 1h
prange_low = 10**(-3) # 30min
prange_high = 5*10**(-2) # 30min
prange_nostep = 201
prange = np.linspace(prange_low,prange_high,prange_nostep)[:, None]

# Probability of exceedance (PoE) levels of two initial samples
init_plevels = np.array([0.001,0.01])

# Low- and high-fidelity signal symbols and units
LFsymb = 'v1'; LFunit = 'Nm' 
HFsymb = 'v2'; HFunit = 'Nm' 

# Only one input case
indcase = 'Hogging'   
# indcase = 'Sagging'


#############
# END INPUT
#############

# Variable symbols and units (to be used in plots generated by AS scipt)
# Format: ['LF variable symbol','HF variable symbol'], same for unit
varsymb = [LFsymb,HFsymb]
varunit = [LFunit,HFunit]

# Derive target exposure duration probability level, accounting for vessel speed and heading
Tpe = calc_T_enc(Tp,Vs,mu)
ne_exp = dur/Tpe

# Read MCS linear and 2nd order true wave time traces from CSV file
AllTrueDat_scr = np.loadtxt(f'{inputfolder}/MARINferry2_MatchedPeaks{indcase}_VBM_{spec}_Hs{Hs}m_Tp{Tp}s_gam{gam}_Vs{Vs}0kn_mu{mu}deg.csv',skiprows=1, delimiter=',')
peaks_lin_all = AllTrueDat_scr[:,1]
peaks_nl_all_scr = AllTrueDat_scr[:,3]
AllTrueDat_true = np.loadtxt(f'{inputfolder}/MARINferry2_PRETTI_peaks_VBM_{spec}_Hs{Hs}m_Tp{Tp}s_gam{gam}_Vs{Vs}0kn_mu{mu}deg.csv',skiprows=1, delimiter=',')
if indcase == 'Sagging':
    peaks_nl_all_true = AllTrueDat_true[:,1]  
elif indcase == 'Hogging':
    peaks_nl_all_true = AllTrueDat_true[:,3]  


# Loop over different acquisition functions
for mm in range(5):
# for mm in [5]:
    
    # Input acquisition functions
    if mm == 0:
        acqHF = 'USMV' # Selected acquisition function for GPR
        acqMF = 'USMV' # Selected acquisition function for MF-GPR
    elif mm == 1:
        acqHF = 'MD'
        acqMF = 'MD'
    elif mm == 2:
        acqHF = 'US'
        acqMF = 'US'
    elif mm == 3:
        acqHF = 'MV'
        acqMF = 'MV'
    elif mm == 4:
        acqHF = 'USPD'
        acqMF = 'USPD'
    elif mm == 5:
        acqHF = 'MVPD'
        acqMF = 'MVPD'
    # elif mm == 6:
    #     acqHF = 'IVR' # for now omit, slow! run from additional file
    #     acqMF = 'IVR' # for now omit, slow! and not properly implemented?
    
    # Run Adaptive Screening
    debugplotswitch = 0
    adaptive_screening(peaks_lin_all,peaks_nl_all_scr,peaks_nl_all_true,Tdata,acqHF,acqMF,maxitHF,maxitMF,prange,init_sampopt,init_plevels,dur,ne_exp,filefolder,figfolder,indcase,varsymb,varunit,debugplotswitch)

    # Close all figures (they are saved in the input file location)
    plt.close('all')
    
# Record end time and run time full script
endtime = time.time()
runtime = endtime-starttime # [s]

