# -*- coding: utf-8 -*-
"""
Created on Wed Sep  6 16:47:18 2023

@author: smvanessen
"""

# Clear all variables
from IPython import get_ipython
get_ipython().magic('reset -sf')

# Standard toolboxes
import numpy as np
import matplotlib.pyplot as plt
import sys
import time

# Import custom functions
sys.path.append('functions')
from functions.distribution_scripts import calc_distribution
from functions.Adaptive_Screening_v9 import adaptive_screening

# Close all figures
plt.close('all')

# Record start time
starttime = time.time()

#############
# START INPUT
#############

# Selected number of considered iterations for GPR
maxitHF = 150

# Selected number of considered iterations for MF-GPR (IVR not implemented, so zero)
maxitMF = 150

# Folder name where input files are stored (files to read)
inputfolder = 'inputfiles_waves'

# Output folder name in which to store files and figures (rel. to present folder)
filefolder = 'out_files_waves_PRADS'
figfolder = 'out_figs_waves_PRADS'

# Target exposure duration [s]
dur = 3600

# Number of seeds. Should comply with the available data, in this case 
# generated using 'Generate_Lin_2nd_MCS_data.py'.
Ns = 50

# Peak primary wave period for all cases
Tp = 11

# Initial sample option (see Adaptive Screening script)
init_sampopt = 'B'

# Probabilty of exceedance (PoE) range for the prediction
prange_low = 5*10**(-4)
prange_high = 1.9*10**(-2)
prange_nostep = 201
prange = np.linspace(prange_low,prange_high,prange_nostep)[:, None]

# Probability of exceedance (PoE) levels of two initial samples
init_plevels = np.array([0.001,0.01])

# Low- and high-fidelity signal symbols and units
LFsymb = 'c1'; LFunit = 'm' 
HFsymb = 'c2'; HFunit = 'm' 

#############
# END INPUT
#############

# Total duration [s]
Tdata = dur*Ns

# Variable symbols and units (to be used in plots generated by AS scipt)
# Format: ['LF variable symbol','HF variable symbol'], same for unit
varsymb = [LFsymb,HFsymb]
varunit = [LFunit,HFunit]

# Loop over both indicator quality cases
for nn in range(2):
# for nn in [0]:
    
    # Input noise (indicator quality) case
    if nn == 0:
        indcase = 'GoodInd'
    elif nn == 1:
        indcase = 'WorseInd'   
      
    # Derive target exposure duration probability level
    ne_exp = dur/Tp
     
    # Read MCS linear and 2nd order true wave time traces from CSV file
    AllTrueDat_scr = np.loadtxt(f'{inputfolder}/Lin2nd_trueLFandHFdata_{indcase}_{Ns}seeds_scr.csv', delimiter=',')
    peaks_lin_all = AllTrueDat_scr[:,1]
    peaks_2nd_all_scr = AllTrueDat_scr[:,3]
    AllTrueDat_true = np.loadtxt(f'{inputfolder}/Lin2nd_trueHFdata_real0_{Ns}seeds.csv', delimiter=',')
    peaks_2nd_all_true = AllTrueDat_true[:,1]  
    
    # Loop over different acquisition functions
    for mm in range(5):
    # for mm in [5]:
        
        # Input acquisition functions
        if mm == 0:
            acqHF = 'USMV' # Selected acquisition function for GPR
            acqMF = 'USMV' # Selected acquisition function for MF-GPR
        elif mm == 1:
            acqHF = 'MD'
            acqMF = 'MD'
        elif mm == 2:
            acqHF = 'US'
            acqMF = 'US'
        elif mm == 3:
            acqHF = 'MV'
            acqMF = 'MV'
        elif mm == 4:
            acqHF = 'USPD'
            acqMF = 'USPD'
        elif mm == 5:
            acqHF = 'MVPD'
            acqMF = 'MVPD'
        # elif mm == 6:
        #     acqHF = 'IVR' # for now omit, slow! run from additional file
        #     acqMF = 'IVR' # for now omit, slow! and not properly implemented?
    
        # Run Adaptive Screening
        debugplotswitch = 0
        adaptive_screening(peaks_lin_all,peaks_2nd_all_scr,peaks_2nd_all_true,Tdata,acqHF,acqMF,maxitHF,maxitMF,prange,init_sampopt,init_plevels,dur,ne_exp,filefolder,figfolder,indcase,varsymb,varunit,debugplotswitch)
    
        # Close all figures (they are saved in the input file location)
        plt.close('all')
    
# Record end time and run time full script
endtime = time.time()
runtime = endtime-starttime # [s]

